module DsSpi (
	clk,
	reset,
	
	DataIn,
	DataOut,
	DataReady,
	ChipSelect,
	
	DSCARD_EEPROMCS,
	DSCARD_EEPROMCLK,
	DSCARD_EEPROMIN,
	DSCARD_EEPROMOUT
);


	input				clk;
	input				reset;
	
	input	[7:0]		DataIn;
	output	[7:0]		DataOut;
	output				DataReady;
	output				ChipSelect;

	input				DSCARD_EEPROMCS;
	input				DSCARD_EEPROMCLK;
	input				DSCARD_EEPROMIN;
	output				DSCARD_EEPROMOUT;
	
	reg		[7:0]		DataOutBuf;
	reg		[7:0]		DataInBuf;
	reg					DataReady;	
	reg		[2:0]		State;
	reg					LastClock;
	reg		[1:0]		EepromClkSync;
	reg		[1:0]		EepromCsSync;
	reg		[1:0]		EepromInSync;

	wire				ShiftOut;
	wire				ShiftIn;
	wire				EepromClk;
	wire				EepromCs;
	wire				EepromIn;
	wire				LastState;
	wire				FirstState;
	wire	[2:0]		NextState;
	wire				DataReq;
	
	assign				EepromClk = EepromClkSync[1];
	assign				EepromCs = EepromCsSync[1];
	assign				EepromIn = EepromInSync[1];
	
	assign				ChipSelect = EepromCs;	
	
	assign				DataOut = DataInBuf;
	assign				ShiftOut = ~EepromClk & LastClock & ~EepromCs;
	assign				ShiftIn = EepromClk & ~LastClock & ~EepromCs;
	assign				FirstState = State == 3'b000;
	assign				LastState = State == 3'b111;
	assign				DataReq = ShiftOut & FirstState;
	assign				NextState = State + 3'b001;

	assign				DSCARD_EEPROMOUT = EepromCs? 1'bZ : DataOutBuf[7];
	
	always @(posedge clk)
	begin
		if (reset)
		begin
			State <= 3'b0;
			LastClock <= 1'b0;
			DataOutBuf <= 8'h0;
			DataInBuf <= 8'h0;
			DataReady <= 1'b0;
			
			EepromClkSync <= 2'b11;
			EepromCsSync <= 2'b11;
			EepromInSync <= 2'b00;
		end
		else
		begin
			DataOutBuf <= DataReq? DataIn : ShiftOut? {DataOutBuf[6:0], 1'b0} : DataOutBuf;
			DataInBuf <= ShiftIn? {DataInBuf[6:0], EepromIn} : DataInBuf;
			State <= ChipSelect? 3'b0 : ShiftIn? NextState : State;
			LastClock <= EepromClk;
			DataReady <= ShiftIn & LastState;
			
			EepromClkSync <= {EepromClkSync[0], DSCARD_EEPROMCLK};
			EepromCsSync <= {EepromCsSync[0], DSCARD_EEPROMCS};
			EepromInSync <= {EepromInSync[0], DSCARD_EEPROMIN};
		end
	end
endmodule

