module UartSlave (
	clk,
	reset,

	address,
	write,
	read,
	writedata,
	readdata,
	
	UartTxClear,
	UartTxFull,
	UartTxData,
	
	UartRxClear,
	UartRxFull,
	UartRxData
);


	input				clk;
	input				reset;

	input				address;
	input				write;
	input				read;
	input		[31:0]	writedata;
	output		[31:0]	readdata;
	
	input				UartTxClear;
	output		[7:0]	UartTxData;
	output				UartTxFull;
	
	input				UartRxFull;
	input		[7:0]	UartRxData;
	output				UartRxClear;
	
	reg					UartRxClear;
	reg					UartTxFull;
	reg			[7:0]	UartTxData;
	// RXCNT:
	// Read: FULL | DATA
	// Write: CLEAR
	
	// TXCNT:
	// Read: FULL | DATA
	// Write: DATA
		
	assign				readdata = address? {23'b0, UartTxFull, UartTxData} : {23'b0, UartRxFull, UartRxData};
	
	always @(posedge clk)
	begin
		if (reset)
		begin
			UartRxClear <= 1'b0;
			UartTxData <= 8'h0;
			UartTxFull <= 1'b0;
		end
		else
		begin
			UartRxClear <= ((~address) & write)? 1'b1 : 1'b0;
			UartTxData <= (address & write)? writedata[7:0] : UartTxData;
			UartTxFull <= (address & write)? 1'b1 : UartTxClear? 1'b0 : UartTxFull;
		end
	end


endmodule

